@@ -17,23 +17,15 @@ your package.
 
 import importlib.metadata
 import os
-from collections.abc import Generator, Sequence
-from importlib.metadata import EntryPoint
-
-
-def get_entry_points() -> Generator[EntryPoint, None, None]:
-    try:
-        eps: Sequence[EntryPoint] = importlib.metadata.entry_points(group="hypothesis")
-    except TypeError:  # pragma: no cover
-        # Load-time selection requires Python >= 3.10.  See also
-        # https://importlib-metadata.readthedocs.io/en/latest/using.html
-        eps = importlib.metadata.entry_points().get("hypothesis", [])
-    yield from eps
 
 
 def run() -> None:
-    if not os.environ.get("HYPOTHESIS_NO_PLUGINS"):
-        for entry in get_entry_points():  # pragma: no cover
-            hook = entry.load()
-            if callable(hook):
-                hook()
+    if os.environ.get("HYPOTHESIS_NO_PLUGINS"):
+        return
+
+    for entry in importlib.metadata.entry_points(
+        group="hypothesis"
+    ):  # pragma: no cover
+        hook = entry.load()
+        if callable(hook):
+            hook()
