@@ -126,6 +126,30 @@ CacheKeyT: TypeAlias = tuple[ChoiceTypeT, tuple[Any, ...]]
 CacheValueT: TypeAlias = tuple[tuple["ConstantT", ...], tuple["ConstantT", ...]]
 CONSTANTS_CACHE: LRUCache[CacheKeyT, CacheValueT] = LRUCache(1024)
 
+
+_constants_integers = (
+    # powers of 2
+    [2**n for n in range(16, 66)]
+    # powers of 10
+    + [10**n for n in range(5, 20)]
+    # factorials
+    + [math.factorial(n) for n in range(9, 21)]
+    # a few primorial numbers https://en.wikipedia.org/wiki/Primorial
+    + [
+        510510,
+        6469693230,
+        304250263527210,
+        32589158477190044730,
+    ]
+)
+_constants_integers.extend(
+    [n - 1 for n in _constants_integers] + [n + 1 for n in _constants_integers]
+)
+_constants_integers.extend([-x for x in _constants_integers])
+
+# arbitrary cutoffs to keep our list bounded
+assert all(50_000 <= abs(n) <= 2**66 for n in _constants_integers)
+
 _constant_floats = (
     [
         0.5,
@@ -170,6 +194,8 @@ _constant_strings = {
     "if",
     "then",
     "else",
+    "__dict__",
+    "__proto__",  # javascript
     # strings which can be interpreted as a number
     "0",
     "1e100",
@@ -191,14 +217,14 @@ _constant_strings = {
     "Ⱥ",
     "Ⱦ",
     # ligatures
-    "æœÆŒﬀʤʨß"
+    "æœÆŒﬀʤʨß",
     # emoticons
     "(╯°□°）╯︵ ┻━┻)",
     # emojis
     "😍",
     "🇺🇸",
     # emoji modifiers
-    "🏻"  # U+1F3FB Light Skin Tone,
+    "🏻",  # U+1F3FB Light Skin Tone,
     "👍🏻",  # 👍 followed by U+1F3FB
     # RTL text
     "الكل في المجمو عة",
@@ -232,7 +258,7 @@ _constant_strings = {
 # we don't actually care what order the constants are sorted in, just that the
 # ordering is deterministic.
 GLOBAL_CONSTANTS = Constants(
-    integers=SortedSet(),
+    integers=SortedSet(_constants_integers),
     floats=SortedSet(_constant_floats, key=float_to_int),
     bytes=SortedSet(),
     strings=SortedSet(_constant_strings),
@@ -729,7 +755,7 @@ class HypothesisProvider(PrimitiveProvider):
 
         # split constants into two pools, so we still have a good chance to draw
         # global constants even if there are many local constants.
-        (global_constants, local_constants) = CONSTANTS_CACHE[key]
+        global_constants, local_constants = CONSTANTS_CACHE[key]
         constants_lists = ([global_constants] if global_constants else []) + (
             [local_constants] if local_constants else []
         )
